#include "AD7142RegMap.h"
#include "Pin Definitions.h"

//----------------------
//Function declarations
//----------------------
//External functions
//==================
void WriteToAD7142ViaSpi(const WORD RegisterAddress, const BYTE NumberOfRegisters, WORD *DataBuffer, const BYTE OffsetInBuffer);
void ReadFromAD7142ViaSpi(const WORD RegisterStartAddress, const BYTE NumberOfRegisters, WORD *DataBuffer, const WORD OffsetInBuffer);
//Local functions
//===============
void ConfigAD7142(void);


//Recal of variables from other C files
//-------------------------------------
extern WORD xdata AD7142Registers[24];


//---------------------
//Function definitions
//---------------------
void ConfigAD7142(void)
{
	WORD xdata ConfigBuffer[8];
	
	//=============================================
	//= Stage 0 - Button 1 = CIN6 Single-Ended(+) =
	//=============================================
	ConfigBuffer[0]=0xEFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0500;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE0_CONNECTION, 8, ConfigBuffer, 0);	//Stage 0 connection registers - Start address in RAM 0x80

	//=============================================
	//= Stage 1 - Button 2 = CIN5 Single-Ended(+) =
	//=============================================
	ConfigBuffer[0]=0xFBFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0A00;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1300;
	ConfigBuffer[7]=1300;
	WriteToAD7142ViaSpi(STAGE1_CONNECTION, 8, ConfigBuffer, 0);	//Stage 1 connection registers - Start address in RAM 0x88

	//=============================================
	//= Stage 2 - Button 3 = CIN4 Single-Ended(+) =
	//=============================================
	ConfigBuffer[0]=0xFEFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0600;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=350;
	ConfigBuffer[5]=350;
	ConfigBuffer[6]=700;
	ConfigBuffer[7]=700;
	WriteToAD7142ViaSpi(STAGE2_CONNECTION, 8, ConfigBuffer, 0);	//Stage 2 connection registers - Start address in RAM 0x90	
	
	//=============================================
	//= Stage 3 - Button 4 = CIN3 Single-Ended(+) =
	//=============================================
	ConfigBuffer[0]=0xFFBF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0300;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=150;
	ConfigBuffer[5]=150;
	ConfigBuffer[6]=250;
	ConfigBuffer[7]=250;
	WriteToAD7142ViaSpi(STAGE3_CONNECTION, 8, ConfigBuffer, 0);	//Stage 3 connection registers - Start address in RAM 0x98

	//=============================================
	//= Stage 4 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE4_CONNECTION, 8, ConfigBuffer, 0);	//Stage 4 connection registers - Start address in RAM 0xA0
	
	//=============================================
	//= Stage 5 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE5_CONNECTION, 8, ConfigBuffer, 0);	//Stage 5 connection registers - Start address in RAM 0xA8
	
	//=============================================
	//= Stage 6 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE6_CONNECTION, 8, ConfigBuffer, 0);	//Stage 6 connection registers - Start address in RAM 0xB0
	
	//=============================================
	//= Stage 7 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE7_CONNECTION, 8, ConfigBuffer, 0);	//Stage 7 connection registers - Start address in RAM 0xB8
	
	//=============================================
	//= Stage 8 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE8_CONNECTION, 8, ConfigBuffer, 0);	//Stage 8 connection registers - Start address in RAM 0xC0
	
	//=============================================
	//= Stage 9 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE9_CONNECTION, 8, ConfigBuffer, 0);	//Stage 9 connection registers - Start address in RAM 0xC8
	
	//=============================================
	//= Stage 10 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE10_CONNECTION, 8, ConfigBuffer, 0);//Stage 10 connection registers - Start address in RAM 0xD0
	
	//=============================================
	//= Stage 11 - Connected to Vbias
	//=============================================
	ConfigBuffer[0]=0xFFFF;
	ConfigBuffer[1]=0x3FFF;
	ConfigBuffer[2]=0x0000;
	ConfigBuffer[3]=0x2626;
	ConfigBuffer[4]=1000;
	ConfigBuffer[5]=1000;
	ConfigBuffer[6]=1500;
	ConfigBuffer[7]=1500;
	WriteToAD7142ViaSpi(STAGE11_CONNECTION, 8, ConfigBuffer, 0);//Stage 11 connection registers - Start address in RAM 0xD8
	
	
	//Initialisation of the first register bank but not the AMBCOMPCTL_REG0
	AD7142Registers[PWR_CONTROL]=0x00B2;		// Low power mode enabled at 200ms - 12 sequences - 256 decimation factor
	WriteToAD7142ViaSpi(PWR_CONTROL, 1, AD7142Registers, PWR_CONTROL);
	
	//Calibration configuration
	AD7142Registers[AMB_COMP_CTRL0]=0x3230;
	AD7142Registers[AMB_COMP_CTRL1]=0x14C8;
	AD7142Registers[AMB_COMP_CTRL2]=0x0832;
	//Interrupt configuration
	AD7142Registers[STAGE_LOW_INT_EN]=0x0000;
	AD7142Registers[STAGE_HIGH_INT_EN]=0x0000;
	AD7142Registers[STAGE_COMPLETE_INT_EN]=0x0001;
	WriteToAD7142ViaSpi(AMB_COMP_CTRL0, 6, AD7142Registers, AMB_COMP_CTRL0);
	
	//Read back ambient compensation register because it has 2 self clearing bits.
	ReadFromAD7142ViaSpi(AMB_COMP_CTRL0, 1, AD7142Registers, AMB_COMP_CTRL0);
	
	//Enable data path for all sequences
	AD7142Registers[STAGE_CAL_EN]=0x0FFF;
	WriteToAD7142ViaSpi(STAGE_CAL_EN, 1, AD7142Registers, STAGE_CAL_EN);
}
