/******************************************************************************/
// Description: In this example, we connected Buttons 1, 2, 3, 4 of the eval boards 
//				to Stage 0, 1, 2, 3 respectively. all other CIN inputs are 
//				conntected to Vbias. We enabled the End of Stage Conversion 
//				Interrupt and read back the 12 ADC result registers and the 3
//				Interrupt Status registers.

// Author: ADI Limerick
// Date: 29/05/2006
/******************************************************************************/

#include <ADUC841.H>	//--> To be replaced with host processor definition file
#include <stdio.h>
#include "Pin Definitions.h"
#include "AD7142regMap.h"


#define SIL_ID					0xE622


//---------------------------------
//Function prototypes
//---------------------------------
//Local functions
//===============
static int ServiceAD7142Isr(void);
void Wait(WORD value);
//External functions
//==================
extern void WriteToAD7142ViaSpi(const WORD RegisterAddress, const BYTE NumberOfRegisters, WORD *DataBuffer, const BYTE OffsetInBuffer);
extern void ReadFromAD7142ViaSpi(const WORD RegisterStartAddress, const BYTE NumberOfRegisters, WORD *DataBuffer, const WORD OffsetInBuffer);
extern void ConfigAD7142(void);


//---------------------------------
//Global variables
//---------------------------------
WORD xdata AD7142Registers[24];	//[0...23 inc]=1st set of registers [0...23 inc]
								//Element[23] = 0x17 = ID register @ 0x17
BYTE xdata EnableRecalCounter;
bit EnableRecalAgain;


//---------------------------------
//void AD7142_Isr() interrupt 0
//---------------------------------
//Interrupt service routine that is executed when the INT0 pin goes low.
//--------------------------------------------------------------------------------
/********************************************************************************************/
/*//////////////////////////////////////////////////////////////////////////////////////////*/
// This function declaration is specific to the ADuC841 and needs to be changed for another //
// host processor. The interrupt vector number may be different 		   					//
/*//////////////////////////////////////////////////////////////////////////////////////////*/
/********************************************************************************************/
static void AD7142_Isr() interrupt 0
{
	EX0=0;
	ServiceAD7142Isr();
	EX0=1;
	return;
}


//---------------------------------
//Main()
//---------------------------------
void main (void)
{	
	/********************************************************************************************/
	/*//////////////////////////////////////////////////////////////////////////////////////////*/
	// Specific to the ADuC841, this section configures the ADuC841 pins as input or as output. //
 	// This needs to be changed for another host processor 		   							  	//
	/*//////////////////////////////////////////////////////////////////////////////////////////*/
	/********************************************************************************************/
	//SPI interface to AD7142
	DOUT=1;				//Set the DOUT pin
	CLK=1;				//Set the CLK pin
	DIN=1;
	CS=1;
	ADUC841_INT0=1;		//Makes sure that the INT0 pin on the ADuC841 is be an input.

	/********************************************************************************/
	/*//////////////////////////////////////////////////////////////////////////////*/
	// Specific to the ADuC841, this line can be removed for another host processor //
	/*//////////////////////////////////////////////////////////////////////////////*/
	/********************************************************************************/
	CFG841 |= 1;//Enable the extended internal RAM
	
	//Read Silicon versions to check communication, It should read 0xE622
	ReadFromAD7142ViaSpi(DEVID, 1, AD7142Registers, DEVID);
	//Configure AD7142
	ConfigAD7142();
	
	/*******************************************************************************************/
	/*/////////////////////////////////////////////////////////////////////////////////////////*/
	// Enable interrupt after software initialisation. This section cannot be placed anywhere  //
	// else. These instructions must be changed for another host processor.					   //
	/*/////////////////////////////////////////////////////////////////////////////////////////*/
	/*******************************************************************************************/
	IT0=0x1;	//INT0 interrupt is negative edged triggered
	EX0=1;		//Enable external interrupt "0" //AD7142 interrupt
	EA=1;		//Enable interrupts
	
	while(1);
}


//ServiceAD7142Isr();
//-----------------------------------------------------------------
// Function called by the AD7142 ISR. Anything that must be 
// executed during the ISR needs to be done here
//-----------------------------------------------------------------
static int ServiceAD7142Isr(void)
{
	//Read interrupt status register to clear the interrupt
	ReadFromAD7142ViaSpi(STAGE_LOW_LIMIT_INT, 3, AD7142Registers, STAGE_LOW_LIMIT_INT);
	//This is how we access the button status
	// AD7142Registers[STAGE_LOW_LIMIT_INT]:
	//======================================
	//	15  14  13  12  11  10  9  8  7  6  5  4  3  2  1  0
	//											  |	 |  |  |----> Button 1
	//											  |  |  |-------> Button 2
	//											  |  |----------> Button 3
	//											  |-------------> Button 4
	
	//=============================================================
	//= Recalibrate the 4 buttons if there is a touch on power up =
	//=============================================================
	if (((AD7142Registers[STAGE_HIGH_LIMIT_INT] & 0x000F) != 0x0000) && 
		((AD7142Registers[STAGE_LOW_LIMIT_INT] & 0x000F) == 0x0000) &&
		(EnableRecalAgain==1))
	{
		//These 2 variables are used to run the re-calibration only once
		EnableRecalAgain=0;
		EnableRecalCounter=0;
		//Recalibrate sensor
		AD7142Registers[AMB_COMP_CTRL0] |= 0x4000;
		WriteToAD7142ViaSpi(AMB_COMP_CTRL0, 1, AD7142Registers, AMB_COMP_CTRL0);
		//Read back
		do
		{
			ReadFromAD7142ViaSpi(AMB_COMP_CTRL0, 1, AD7142Registers, AMB_COMP_CTRL0);
		}while((AD7142Registers[AMB_COMP_CTRL0] & 0x4000) == 0x4000);
	}
	//Once the recalibration has occured, we must wait for 3 interrupts before to 
	//enable it again.
	if (EnableRecalCounter<4 && EnableRecalAgain==0)
		EnableRecalCounter++;
	if (EnableRecalCounter==3 && EnableRecalAgain==0)
		EnableRecalAgain=1;
	
	//Read ADC Values
	ReadFromAD7142ViaSpi(ADCRESULT_S0, 12, AD7142Registers, ADCRESULT_S0);
	//This is how we access the ADC Values
	//AD7142Registers[ADCRESULT_S0] corresponds to ADC Value of Stage 0
	//AD7142Registers[ADCRESULT_S1] corresponds to ADC Value of Stage 1
	//AD7142Registers[ADCRESULT_S2] corresponds to ADC Value of Stage 2
	//AD7142Registers[ADCRESULT_S3] corresponds to ADC Value of Stage 3
	//AD7142Registers[ADCRESULT_S4] corresponds to ADC Value of Stage 4
	//AD7142Registers[ADCRESULT_S5] corresponds to ADC Value of Stage 5
	//AD7142Registers[ADCRESULT_S6] corresponds to ADC Value of Stage 6
	//AD7142Registers[ADCRESULT_S7] corresponds to ADC Value of Stage 7
	//AD7142Registers[ADCRESULT_S8] corresponds to ADC Value of Stage 8
	//AD7142Registers[ADCRESULT_S9] corresponds to ADC Value of Stage 9
	//AD7142Registers[ADCRESULT_S10] corresponds to ADC Value of Stage 10
	//AD7142Registers[ADCRESULT_S11] corresponds to ADC Value of Stage 11
}


//---------------------------------
//Wait();
//---------------------------------
//This is just a simple delay routine.
//--------------------------------------------------------------------------------
void Wait(WORD value)
{
	while (value>0)
		value--;
}

